# coding: utf-8

"""
    Lightly API

    Lightly.ai enables you to do self-supervised learning in an easy and intuitive way. The lightly.ai OpenAPI spec defines how one can interact with our REST API to unleash the full potential of lightly.ai  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: support@lightly.ai
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from lightly.openapi_generated.swagger_client.configuration import Configuration


class ActiveLearningScoreData(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'MongoObjectID',
        'tag_id': 'MongoObjectID',
        'score_type': 'ActiveLearningScoreType',
        'scores': 'ActiveLearningScores',
        'created_at': 'Timestamp'
    }

    attribute_map = {
        'id': 'id',
        'tag_id': 'tagId',
        'score_type': 'scoreType',
        'scores': 'scores',
        'created_at': 'createdAt'
    }

    def __init__(self, id=None, tag_id=None, score_type=None, scores=None, created_at=None, _configuration=None):  # noqa: E501
        """ActiveLearningScoreData - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._id = None
        self._tag_id = None
        self._score_type = None
        self._scores = None
        self._created_at = None
        self.discriminator = None

        self.id = id
        self.tag_id = tag_id
        self.score_type = score_type
        self.scores = scores
        self.created_at = created_at

    @property
    def id(self):
        """Gets the id of this ActiveLearningScoreData.  # noqa: E501


        :return: The id of this ActiveLearningScoreData.  # noqa: E501
        :rtype: MongoObjectID
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ActiveLearningScoreData.


        :param id: The id of this ActiveLearningScoreData.  # noqa: E501
        :type: MongoObjectID
        """
        if self._configuration.client_side_validation and id is None:
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def tag_id(self):
        """Gets the tag_id of this ActiveLearningScoreData.  # noqa: E501


        :return: The tag_id of this ActiveLearningScoreData.  # noqa: E501
        :rtype: MongoObjectID
        """
        return self._tag_id

    @tag_id.setter
    def tag_id(self, tag_id):
        """Sets the tag_id of this ActiveLearningScoreData.


        :param tag_id: The tag_id of this ActiveLearningScoreData.  # noqa: E501
        :type: MongoObjectID
        """
        if self._configuration.client_side_validation and tag_id is None:
            raise ValueError("Invalid value for `tag_id`, must not be `None`")  # noqa: E501

        self._tag_id = tag_id

    @property
    def score_type(self):
        """Gets the score_type of this ActiveLearningScoreData.  # noqa: E501


        :return: The score_type of this ActiveLearningScoreData.  # noqa: E501
        :rtype: ActiveLearningScoreType
        """
        return self._score_type

    @score_type.setter
    def score_type(self, score_type):
        """Sets the score_type of this ActiveLearningScoreData.


        :param score_type: The score_type of this ActiveLearningScoreData.  # noqa: E501
        :type: ActiveLearningScoreType
        """
        if self._configuration.client_side_validation and score_type is None:
            raise ValueError("Invalid value for `score_type`, must not be `None`")  # noqa: E501

        self._score_type = score_type

    @property
    def scores(self):
        """Gets the scores of this ActiveLearningScoreData.  # noqa: E501


        :return: The scores of this ActiveLearningScoreData.  # noqa: E501
        :rtype: ActiveLearningScores
        """
        return self._scores

    @scores.setter
    def scores(self, scores):
        """Sets the scores of this ActiveLearningScoreData.


        :param scores: The scores of this ActiveLearningScoreData.  # noqa: E501
        :type: ActiveLearningScores
        """
        if self._configuration.client_side_validation and scores is None:
            raise ValueError("Invalid value for `scores`, must not be `None`")  # noqa: E501

        self._scores = scores

    @property
    def created_at(self):
        """Gets the created_at of this ActiveLearningScoreData.  # noqa: E501


        :return: The created_at of this ActiveLearningScoreData.  # noqa: E501
        :rtype: Timestamp
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this ActiveLearningScoreData.


        :param created_at: The created_at of this ActiveLearningScoreData.  # noqa: E501
        :type: Timestamp
        """
        if self._configuration.client_side_validation and created_at is None:
            raise ValueError("Invalid value for `created_at`, must not be `None`")  # noqa: E501

        self._created_at = created_at

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ActiveLearningScoreData, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ActiveLearningScoreData):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ActiveLearningScoreData):
            return True

        return self.to_dict() != other.to_dict()



""" MoCo Model """

# Copyright (c) 2020. Lightly AG and its affiliates.
# All Rights Reserved

import warnings

import torch
import torch.nn as nn

from lightly.models._momentum import _MomentumEncoderMixin
from lightly.models.modules import MoCoProjectionHead


class MoCo(nn.Module, _MomentumEncoderMixin):
    """Implementation of the MoCo (Momentum Contrast)[0] architecture.

    Recommended loss: :py:class:`lightly.loss.ntx_ent_loss.NTXentLoss` with
    a memory bank.

    [0] MoCo, 2020, https://arxiv.org/abs/1911.05722

    Attributes:
        backbone:
            Backbone model to extract features from images.
        num_ftrs:
            Dimension of the embedding (before the projection head).
        out_dim:
            Dimension of the output (after the projection head).
        m:
            Momentum for momentum update of the key-encoder.

    """

    def __init__(
        self,
        backbone: nn.Module,
        num_ftrs: int = 32,
        out_dim: int = 128,
        m: float = 0.999,
        batch_shuffle: bool = False,
    ):

        super(MoCo, self).__init__()

        self.backbone = backbone
        self.projection_head = MoCoProjectionHead(num_ftrs, num_ftrs, out_dim)
        self.momentum_features = None
        self.momentum_projection_head = None

        self.m = m
        self.batch_shuffle = batch_shuffle

        # initialize momentum features and momentum projection head
        self._init_momentum_encoder()

        warnings.warn(
            Warning(
                "The high-level building block MoCo will be deprecated in version 1.3.0. "
                + "Use low-level building blocks instead. "
                + "See https://docs.lightly.ai/self-supervised-learning/lightly.models.html for more information"
            ),
            PendingDeprecationWarning,
        )

    def forward(
        self, x0: torch.Tensor, x1: torch.Tensor = None, return_features: bool = False
    ):
        """Embeds and projects the input image.

        Performs the momentum update, extracts features with the backbone and
        applies the projection head to the output space. If both x0 and x1 are
        not None, both will be passed through the backbone and projection head.
        If x1 is None, only x0 will be forwarded.

        Args:
            x0:
                Tensor of shape bsz x channels x W x H.
            x1:
                Tensor of shape bsz x channels x W x H.
            return_features:
                Whether or not to return the intermediate features backbone(x).

        Returns:
            The output projection of x0 and (if x1 is not None) the output
            projection of x1. If return_features is True, the output for each x
            is a tuple (out, f) where f are the features before the projection
            head.

        Examples:
            >>> # single input, single output
            >>> out = model(x)
            >>>
            >>> # single input with return_features=True
            >>> out, f = model(x, return_features=True)
            >>>
            >>> # two inputs, two outputs
            >>> out0, out1 = model(x0, x1)
            >>>
            >>> # two inputs, two outputs with return_features=True
            >>> (out0, f0), (out1, f1) = model(x0, x1, return_features=True)

        """
        self._momentum_update(self.m)

        # forward pass of first input x0
        f0 = self.backbone(x0).flatten(start_dim=1)
        out0 = self.projection_head(f0)

        # append features if requested
        if return_features:
            out0 = (out0, f0)

        # return out0 if x1 is None
        if x1 is None:
            return out0

        # forward pass of second input x1
        with torch.no_grad():

            # shuffle for batchnorm
            if self.batch_shuffle:
                x1, shuffle = self._batch_shuffle(x1)

            # run x1 through momentum encoder
            f1 = self.momentum_backbone(x1).flatten(start_dim=1)
            out1 = self.momentum_projection_head(f1).detach()

            # unshuffle for batchnorm
            if self.batch_shuffle:
                f1 = self._batch_unshuffle(f1, shuffle)
                out1 = self._batch_unshuffle(out1, shuffle)

            # append features if requested
            if return_features:
                out1 = (out1, f1)

        return out0, out1



import warnings
from io import IOBase
from typing import *
import platform
import os

import requests

from lightly.api.api_workflow_predictions import _PredictionsMixin
from lightly.api.api_workflow_tags import _TagsMixin
from requests import Response

from lightly.__init__ import __version__
from lightly.api.api_workflow_collaboration import _CollaborationMixin
from lightly.api.api_workflow_compute_worker import _ComputeWorkerMixin
from lightly.api.api_workflow_datasets import _DatasetsMixin
from lightly.api.api_workflow_datasources import _DatasourcesMixin
from lightly.api.api_workflow_download_dataset import _DownloadDatasetMixin
from lightly.api.api_workflow_selection import _SelectionMixin
from lightly.api.api_workflow_upload_dataset import _UploadDatasetMixin
from lightly.api.api_workflow_upload_embeddings import _UploadEmbeddingsMixin
from lightly.api.api_workflow_upload_metadata import _UploadCustomMetadataMixin
from lightly.api.utils import DatasourceType, get_signed_url_destination, get_api_client_configuration
from lightly.api.version_checking import is_compatible_version, LightlyAPITimeoutException
from lightly.openapi_generated.swagger_client.api.collaboration_api import CollaborationApi
from lightly.openapi_generated.swagger_client import ScoresApi, QuotaApi, MetaDataConfigurationsApi, PredictionsApi, DatasetCreator
from lightly.openapi_generated.swagger_client.api.datasets_api import \
    DatasetsApi
from lightly.openapi_generated.swagger_client.api.datasources_api import \
    DatasourcesApi
from lightly.openapi_generated.swagger_client.api.docker_api import DockerApi
from lightly.openapi_generated.swagger_client.api.embeddings_api import \
    EmbeddingsApi
from lightly.openapi_generated.swagger_client.api.jobs_api import JobsApi
from lightly.openapi_generated.swagger_client.api.mappings_api import \
    MappingsApi
from lightly.openapi_generated.swagger_client.api.samples_api import SamplesApi
from lightly.openapi_generated.swagger_client.api.samplings_api import \
    SamplingsApi
from lightly.openapi_generated.swagger_client.api.tags_api import TagsApi
from lightly.openapi_generated.swagger_client.api_client import ApiClient
from lightly.openapi_generated.swagger_client.models.dataset_data import \
    DatasetData
from lightly.utils.reordering import sort_items_by_keys

# Env variable for server side encryption on S3
LIGHTLY_S3_SSE_KMS_KEY = 'LIGHTLY_S3_SSE_KMS_KEY' 

class ApiWorkflowClient(_UploadEmbeddingsMixin,
                        _SelectionMixin,
                        _UploadDatasetMixin,
                        _DownloadDatasetMixin,
                        _DatasetsMixin,
                        _UploadCustomMetadataMixin,
                        _TagsMixin,
                        _DatasourcesMixin,
                        _ComputeWorkerMixin,
                        _CollaborationMixin,
                        _PredictionsMixin,
                        ):
    """Provides a uniform interface to communicate with the api 
    
    The APIWorkflowClient is used to communicaate with the Lightly API. The client
    can run also more complex workflows which include multiple API calls at once.
    
    The client can be used in combination with the active learning agent. 

    Args:
        token:
            the token of the user, provided in webapp
        dataset_id:
            the id of the dataset, provided in webapp. \
            If it is not set, but used by a workflow, \
            the last modfied dataset is taken by default.
        embedding_id:
            the id of the embedding to use. If it is not set, \
            but used by a workflow, the newest embedding is taken by default
        dataset_creator:
            Telling from where the dataset is created.
    """

    def __init__(
        self,
        token: Optional[str] = None,
        dataset_id: Optional[str] = None,
        embedding_id: Optional[str] = None,
        dataset_creator: str = DatasetCreator.USER_PIP,
    ):

        try:
            if not is_compatible_version(__version__):
                warnings.warn(
                    UserWarning((f"Incompatible version of lightly pip package. "
                                f"Please upgrade to the latest version "
                                f"to be able to access the api.")
                    )
                )
        except LightlyAPITimeoutException:
            pass

        configuration = get_api_client_configuration(token=token)
        self.api_client = ApiClient(configuration=configuration)
        self.api_client.user_agent = f"Lightly/{__version__} ({platform.system()}/{platform.release()}; {platform.platform()}; {platform.processor()};) python/{platform.python_version()}"
        self.set_request_timeout(DEFAULT_API_TIMEOUT)

        self.token = configuration.api_key["token"]
        if dataset_id is not None:
            self._dataset_id = dataset_id
        if embedding_id is not None:
            self.embedding_id = embedding_id
        self._dataset_creator = dataset_creator

        self._collaboration_api = CollaborationApi(api_client=self.api_client)
        self._compute_worker_api = DockerApi(api_client=self.api_client)
        self._datasets_api = DatasetsApi(api_client=self.api_client)
        self._datasources_api = DatasourcesApi(api_client=self.api_client)
        self._selection_api = SamplingsApi(api_client=self.api_client)
        self._jobs_api = JobsApi(api_client=self.api_client)
        self._tags_api = TagsApi(api_client=self.api_client)
        self._embeddings_api = EmbeddingsApi(api_client=self.api_client)
        self._mappings_api = MappingsApi(api_client=self.api_client)
        self._scores_api = ScoresApi(api_client=self.api_client)
        self._samples_api = SamplesApi(api_client=self.api_client)
        self._quota_api = QuotaApi(api_client=self.api_client)
        self._metadata_configurations_api = \
            MetaDataConfigurationsApi(api_client=self.api_client)
        self._predictions_api = PredictionsApi(api_client=self.api_client)

    @property
    def dataset_id(self) -> str:
        '''The current dataset_id.

        If the dataset_id is set, it is returned.
        If it is not set, then the dataset_id of the last modified dataset is selected.
        ''' 
        try:
            return self._dataset_id
        except AttributeError:
            all_datasets: List[DatasetData] = self.get_datasets()
            datasets_sorted = sorted(all_datasets, key=lambda dataset: dataset.last_modified_at)
            last_modified_dataset = datasets_sorted[-1]
            self._dataset_id = last_modified_dataset.id
            warnings.warn(UserWarning(f"Dataset has not been specified, "
                                      f"taking the last modified dataset {last_modified_dataset.name} as default dataset."))
            return self._dataset_id

    @dataset_id.setter
    def dataset_id(self, dataset_id: str):
        """Sets the current dataset id for the client.
        
        Args:
            dataset_id:
                The new dataset id.

        Raises:
            ValueError if the dataset id does not exist.
        """
        if not self.dataset_exists(dataset_id):
            raise ValueError(
                f"A dataset with the id {dataset_id} does not exist on the web"
                f"platform."
            )
        self._dataset_id = dataset_id
        

    def _order_list_by_filenames(
            self, filenames_for_list: List[str],
            list_to_order: List[object]
    ) -> List[object]:
        """Orders a list such that it is in the order of the filenames specified on the server.

        Args:
            filenames_for_list:
                The filenames of samples in a specific order
            list_to_order:
                Some values belonging to the samples

        Returns:
            The list reordered.
            The same reorder applied on the filenames_for_list would put them
            in the order of the filenames in self.filenames_on_server.
            every filename in self.filenames_on_server must be in the
            filenames_for_list.

        """
        filenames_on_server = self.get_filenames()
        list_ordered = sort_items_by_keys(
            filenames_for_list, list_to_order, filenames_on_server
        )
        return list_ordered

    def get_filenames(self) -> List[str]:
        """Downloads the list of filenames from the server.

        This is an expensive operation, especially for large datasets.
        """
        filenames_on_server = self._mappings_api. \
            get_sample_mappings_by_dataset_id(dataset_id=self.dataset_id, field="fileName")
        return filenames_on_server

    def upload_file_with_signed_url(
        self,
        file: IOBase,
        signed_write_url: str,
        headers: Optional[Dict] = None,
        session: Optional[requests.Session] = None,
    ) -> Response:
        """Uploads a file to a url via a put request.

        Args:
            file:
                The file to upload.
            signed_write_url:
                The url to upload the file to. As no authorization is used,
                the url must be a signed write url.
            headers:
                Specific headers for the request.
            session:
                Optional requests session used to upload the file.

        Returns:
            The response of the put request, usually a 200 for the success case.

        """

        # check to see if server side encryption for S3 is desired
        # see https://docs.aws.amazon.com/AmazonS3/latest/userguide/UsingServerSideEncryption.html
        # see https://docs.aws.amazon.com/AmazonS3/latest/userguide/UsingKMSEncryption.html
        lightly_s3_sse_kms_key = os.environ.get(LIGHTLY_S3_SSE_KMS_KEY, '').strip()
        # Only set s3 related headers when we are talking with s3
        if get_signed_url_destination(signed_write_url)==DatasourceType.S3 and lightly_s3_sse_kms_key:
            if headers is None:
                headers = {}
            # don't override previously set SSE
            if 'x-amz-server-side-encryption' not in headers:
                if lightly_s3_sse_kms_key.lower() == 'true':
                    # enable SSE with the key of amazon
                    headers['x-amz-server-side-encryption'] = 'AES256'
                else:
                    # enable SSE with specific customer KMS key
                    headers['x-amz-server-side-encryption'] = 'aws:kms'
                    headers['x-amz-server-side-encryption-aws-kms-key-id'] = lightly_s3_sse_kms_key

        # start requests session and make put request
        sess = session or requests
        if headers is not None:
            response = sess.put(signed_write_url, data=file, headers=headers)
        else:
            response = sess.put(signed_write_url, data=file)
        response.raise_for_status()
        return response

    def set_request_timeout(self, timeout: Union[int, Tuple[int, int]]):
        """Sets a default timeout for all api requests.

        Args:
            timeout:
                Timeout in seconds. Is either a single total_timeout value or a
                (connect_timeout, read_timeout) tuple. 
                See https://urllib3.readthedocs.io/en/stable/reference/urllib3.util.html?highlight=timeout#urllib3.util.Timeout
                for details on the different values.
        """
        set_api_client_request_timeout(client=self.api_client, timeout=timeout)


DEFAULT_API_TIMEOUT = 60 * 3 # seconds

def set_api_client_request_timeout(
    client: ApiClient, 
    timeout: Union[int, Tuple[int, int]] = DEFAULT_API_TIMEOUT,
):
    """Sets a default timeout for all requests with the client.

    This function patches the request method of the api client. This is 
    necessary because the swagger api client does not respect any timeouts 
    configured by urllib3. Instead it expects a timeout to be passed with every
    request. Code here: https://github.com/lightly-ai/lightly/blob/ffbd32fe82f76b37c8ac497640355314474bfc3b/lightly/openapi_generated/swagger_client/rest.py#L141-L148

    Args:
        client:
            Api client on which the timeout is applied.
        timeout:
            Timeout in seconds. Is either a single total_timeout value or a
            (connect_timeout, read_timeout) tuple. 
            See https://urllib3.readthedocs.io/en/stable/reference/urllib3.util.html?highlight=timeout#urllib3.util.Timeout
            for details on the different values.

    """
    request_fn = client.rest_client.request

    def new_request_fn(*args, **kwargs):
        request_timeout = kwargs['_request_timeout']
        if request_timeout is None:
            kwargs['_request_timeout'] = timeout
        return request_fn(*args, **kwargs)

    client.rest_client.request = new_request_fn



""" Momentum Encoder """

# Copyright (c) 2020. Lightly AG and its affiliates.
# All Rights Reserved

import copy

import torch
import torch.nn as nn


def _deactivate_requires_grad(params):
    """Deactivates the requires_grad flag for all parameters.

    """
    for param in params:
        param.requires_grad = False


def _do_momentum_update(prev_params, params, m):
    """Updates the weights of the previous parameters.

    """
    for prev_param, param in zip(prev_params, params):
        prev_param.data = prev_param.data * m + param.data * (1. - m)


class _MomentumEncoderMixin:
    """Mixin to provide momentum encoder functionalities.

    Provides the following functionalities:
        - Momentum encoder initialization.
        - Momentum updates.
        - Batch shuffling and unshuffling.

    To make use of the mixin, simply inherit from it:

    >>> class MyMoCo(nn.Module, _MomentumEncoderMixin):
    >>>
    >>>     def __init__(self, backbone):
    >>>         super(MyMoCo, self).__init__()
    >>>
    >>>         self.backbone = backbone
    >>>         self.projection_head = get_projection_head()
    >>>
    >>>         # initialize momentum_backbone and momentum_projection_head
    >>>         self._init_momentum_encoder()
    >>>
    >>>     def forward(self, x: torch.Tensor):
    >>>
    >>>         # do the momentum update
    >>>         self._momentum_update(0.999)
    >>>
    >>>         # use momentum backbone
    >>>         y = self.momentum_backbone(x)
    >>>         y = self.momentum_projection_head(y)

    """

    m: float
    backbone: nn.Module
    projection_head: nn.Module
    momentum_backbone: nn.Module
    momentum_projection_head: nn.Module

    def _init_momentum_encoder(self):
        """Initializes momentum backbone and a momentum projection head.

        """
        assert self.backbone is not None
        assert self.projection_head is not None

        self.momentum_backbone = copy.deepcopy(self.backbone)
        self.momentum_projection_head = copy.deepcopy(self.projection_head)
        
        _deactivate_requires_grad(self.momentum_backbone.parameters())
        _deactivate_requires_grad(self.momentum_projection_head.parameters())

    @torch.no_grad()
    def _momentum_update(self, m: float = 0.999):
        """Performs the momentum update for the backbone and projection head.

        """
        _do_momentum_update(
            self.momentum_backbone.parameters(),
            self.backbone.parameters(),
            m=m,
        )
        _do_momentum_update(
            self.momentum_projection_head.parameters(),
            self.projection_head.parameters(),
            m=m,
        )

    @torch.no_grad()
    def _batch_shuffle(self, batch: torch.Tensor):
        """Returns the shuffled batch and the indices to undo.

        """
        batch_size = batch.shape[0]
        shuffle = torch.randperm(batch_size, device=batch.device)
        return batch[shuffle], shuffle

    @torch.no_grad()
    def _batch_unshuffle(self, batch: torch.Tensor, shuffle: torch.Tensor):
        """Returns the unshuffled batch.

        """
        unshuffle = torch.argsort(shuffle)
        return batch[unshuffle]



""" Check for Corrupt Images """

# Copyright (c) 2020. Lightly AG and its affiliates.
# All Rights Reserved

import os
from typing import *
from PIL import Image
from PIL import UnidentifiedImageError
import tqdm.contrib.concurrent as concurrent
from lightly.data import LightlyDataset


def check_images(data_dir: str) -> Tuple[List[str], List[str]]:
    '''Iterate through a directory of images and find corrupt images

    Args:
        data_dir: Path to the directory containing the images

    Returns:
        (healthy_images, corrupt_images)
    '''
    dataset = LightlyDataset(input_dir=data_dir)
    filenames = dataset.get_filenames()

    def _is_corrupt(filename):
        try:
            image = Image.open(
                os.path.join(data_dir, filename)
            )
            image.load()
        except (IOError, UnidentifiedImageError):
            return True
        else:
            return False

    mapped = concurrent.thread_map(
        _is_corrupt,
        filenames,
        chunksize=min(32, len(filenames))
    )
    healthy_images = [f for f, is_corrupt
                      in zip(filenames, mapped) if not is_corrupt]
    corrupt_images = [f for f, is_corrupt
                      in zip(filenames, mapped) if is_corrupt]
    return healthy_images, corrupt_images



# coding: utf-8

"""
    Lightly API

    Lightly.ai enables you to do self-supervised learning in an easy and intuitive way. The lightly.ai OpenAPI spec defines how one can interact with our REST API to unleash the full potential of lightly.ai  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: support@lightly.ai
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from lightly.openapi_generated.swagger_client.configuration import Configuration


class DockerWorkerConfigV2LightlyCollate(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'input_size': 'int',
        'cj_prob': 'float',
        'cj_bright': 'float',
        'cj_contrast': 'float',
        'cj_sat': 'float',
        'cj_hue': 'float',
        'min_scale': 'float',
        'random_gray_scale': 'float',
        'gaussian_blur': 'float',
        'kernel_size': 'float',
        'vf_prob': 'float',
        'hf_prob': 'float',
        'rr_prob': 'float'
    }

    attribute_map = {
        'input_size': 'inputSize',
        'cj_prob': 'cjProb',
        'cj_bright': 'cjBright',
        'cj_contrast': 'cjContrast',
        'cj_sat': 'cjSat',
        'cj_hue': 'cjHue',
        'min_scale': 'minScale',
        'random_gray_scale': 'randomGrayScale',
        'gaussian_blur': 'gaussianBlur',
        'kernel_size': 'kernelSize',
        'vf_prob': 'vfProb',
        'hf_prob': 'hfProb',
        'rr_prob': 'rrProb'
    }

    def __init__(self, input_size=None, cj_prob=None, cj_bright=None, cj_contrast=None, cj_sat=None, cj_hue=None, min_scale=None, random_gray_scale=None, gaussian_blur=None, kernel_size=None, vf_prob=None, hf_prob=None, rr_prob=None, _configuration=None):  # noqa: E501
        """DockerWorkerConfigV2LightlyCollate - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._input_size = None
        self._cj_prob = None
        self._cj_bright = None
        self._cj_contrast = None
        self._cj_sat = None
        self._cj_hue = None
        self._min_scale = None
        self._random_gray_scale = None
        self._gaussian_blur = None
        self._kernel_size = None
        self._vf_prob = None
        self._hf_prob = None
        self._rr_prob = None
        self.discriminator = None

        if input_size is not None:
            self.input_size = input_size
        if cj_prob is not None:
            self.cj_prob = cj_prob
        if cj_bright is not None:
            self.cj_bright = cj_bright
        if cj_contrast is not None:
            self.cj_contrast = cj_contrast
        if cj_sat is not None:
            self.cj_sat = cj_sat
        if cj_hue is not None:
            self.cj_hue = cj_hue
        if min_scale is not None:
            self.min_scale = min_scale
        if random_gray_scale is not None:
            self.random_gray_scale = random_gray_scale
        if gaussian_blur is not None:
            self.gaussian_blur = gaussian_blur
        if kernel_size is not None:
            self.kernel_size = kernel_size
        if vf_prob is not None:
            self.vf_prob = vf_prob
        if hf_prob is not None:
            self.hf_prob = hf_prob
        if rr_prob is not None:
            self.rr_prob = rr_prob

    @property
    def input_size(self):
        """Gets the input_size of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501


        :return: The input_size of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :rtype: int
        """
        return self._input_size

    @input_size.setter
    def input_size(self, input_size):
        """Sets the input_size of this DockerWorkerConfigV2LightlyCollate.


        :param input_size: The input_size of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :type: int
        """

        self._input_size = input_size

    @property
    def cj_prob(self):
        """Gets the cj_prob of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501


        :return: The cj_prob of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :rtype: float
        """
        return self._cj_prob

    @cj_prob.setter
    def cj_prob(self, cj_prob):
        """Sets the cj_prob of this DockerWorkerConfigV2LightlyCollate.


        :param cj_prob: The cj_prob of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :type: float
        """

        self._cj_prob = cj_prob

    @property
    def cj_bright(self):
        """Gets the cj_bright of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501


        :return: The cj_bright of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :rtype: float
        """
        return self._cj_bright

    @cj_bright.setter
    def cj_bright(self, cj_bright):
        """Sets the cj_bright of this DockerWorkerConfigV2LightlyCollate.


        :param cj_bright: The cj_bright of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :type: float
        """

        self._cj_bright = cj_bright

    @property
    def cj_contrast(self):
        """Gets the cj_contrast of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501


        :return: The cj_contrast of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :rtype: float
        """
        return self._cj_contrast

    @cj_contrast.setter
    def cj_contrast(self, cj_contrast):
        """Sets the cj_contrast of this DockerWorkerConfigV2LightlyCollate.


        :param cj_contrast: The cj_contrast of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :type: float
        """

        self._cj_contrast = cj_contrast

    @property
    def cj_sat(self):
        """Gets the cj_sat of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501


        :return: The cj_sat of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :rtype: float
        """
        return self._cj_sat

    @cj_sat.setter
    def cj_sat(self, cj_sat):
        """Sets the cj_sat of this DockerWorkerConfigV2LightlyCollate.


        :param cj_sat: The cj_sat of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :type: float
        """

        self._cj_sat = cj_sat

    @property
    def cj_hue(self):
        """Gets the cj_hue of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501


        :return: The cj_hue of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :rtype: float
        """
        return self._cj_hue

    @cj_hue.setter
    def cj_hue(self, cj_hue):
        """Sets the cj_hue of this DockerWorkerConfigV2LightlyCollate.


        :param cj_hue: The cj_hue of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :type: float
        """

        self._cj_hue = cj_hue

    @property
    def min_scale(self):
        """Gets the min_scale of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501


        :return: The min_scale of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :rtype: float
        """
        return self._min_scale

    @min_scale.setter
    def min_scale(self, min_scale):
        """Sets the min_scale of this DockerWorkerConfigV2LightlyCollate.


        :param min_scale: The min_scale of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :type: float
        """

        self._min_scale = min_scale

    @property
    def random_gray_scale(self):
        """Gets the random_gray_scale of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501


        :return: The random_gray_scale of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :rtype: float
        """
        return self._random_gray_scale

    @random_gray_scale.setter
    def random_gray_scale(self, random_gray_scale):
        """Sets the random_gray_scale of this DockerWorkerConfigV2LightlyCollate.


        :param random_gray_scale: The random_gray_scale of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :type: float
        """

        self._random_gray_scale = random_gray_scale

    @property
    def gaussian_blur(self):
        """Gets the gaussian_blur of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501


        :return: The gaussian_blur of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :rtype: float
        """
        return self._gaussian_blur

    @gaussian_blur.setter
    def gaussian_blur(self, gaussian_blur):
        """Sets the gaussian_blur of this DockerWorkerConfigV2LightlyCollate.


        :param gaussian_blur: The gaussian_blur of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :type: float
        """

        self._gaussian_blur = gaussian_blur

    @property
    def kernel_size(self):
        """Gets the kernel_size of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501


        :return: The kernel_size of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :rtype: float
        """
        return self._kernel_size

    @kernel_size.setter
    def kernel_size(self, kernel_size):
        """Sets the kernel_size of this DockerWorkerConfigV2LightlyCollate.


        :param kernel_size: The kernel_size of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :type: float
        """

        self._kernel_size = kernel_size

    @property
    def vf_prob(self):
        """Gets the vf_prob of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501


        :return: The vf_prob of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :rtype: float
        """
        return self._vf_prob

    @vf_prob.setter
    def vf_prob(self, vf_prob):
        """Sets the vf_prob of this DockerWorkerConfigV2LightlyCollate.


        :param vf_prob: The vf_prob of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :type: float
        """

        self._vf_prob = vf_prob

    @property
    def hf_prob(self):
        """Gets the hf_prob of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501


        :return: The hf_prob of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :rtype: float
        """
        return self._hf_prob

    @hf_prob.setter
    def hf_prob(self, hf_prob):
        """Sets the hf_prob of this DockerWorkerConfigV2LightlyCollate.


        :param hf_prob: The hf_prob of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :type: float
        """

        self._hf_prob = hf_prob

    @property
    def rr_prob(self):
        """Gets the rr_prob of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501


        :return: The rr_prob of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :rtype: float
        """
        return self._rr_prob

    @rr_prob.setter
    def rr_prob(self, rr_prob):
        """Sets the rr_prob of this DockerWorkerConfigV2LightlyCollate.


        :param rr_prob: The rr_prob of this DockerWorkerConfigV2LightlyCollate.  # noqa: E501
        :type: float
        """

        self._rr_prob = rr_prob

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(DockerWorkerConfigV2LightlyCollate, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DockerWorkerConfigV2LightlyCollate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, DockerWorkerConfigV2LightlyCollate):
            return True

        return self.to_dict() != other.to_dict()



from typing import List, Tuple

from lightly.active_learning.utils import BoundingBox


def read_yolo_label_file(filepath: str, padding: float, separator: str = ' ') -> Tuple[List[int], List[BoundingBox]]:
    """Reads a file in the yolo file format

    Args:
        filepath:
            The path to the yolo file, usually a .txt file.
        padding:
            The relative passepartout / padding to be added around the bounding box
        separator:
            The separator character between the 5 values (class id, x, y, w, h) per row in the label file.

    Returns:
        Two lists, each with one element per row in the label file:
            The class indices.
            The bounding boxes.

    """
    with open(filepath, 'r') as f:
        lines = f.readlines()

    class_indices = []
    bounding_boxes = []
    for line in lines:
        values = line.split(sep=separator)
        class_id, x_norm, y_norm, w_norm, h_norm = (float(val) for val in values)
        class_id = int(class_id)
        class_indices.append(class_id)

        w_norm *= 1+padding
        h_norm *= 1+padding
        bbox = BoundingBox.from_yolo_label(x_norm, y_norm, w_norm, h_norm)
        bounding_boxes.append(bbox)
    return class_indices, bounding_boxes



"""The lightly.loss.regularizer package provides regularizers for self-supervised learning. """


# Copyright (c) 2020. Lightly AG and its affiliates.
# All Rights Reserved

from lightly.loss.regularizer.co2 import CO2Regularizer



# -*- coding: utf-8 -*-
"""**Lightly Train:** Train a self-supervised model from the command-line.

This module contains the entrypoint for the **lightly-train**
command-line interface.
"""

# Copyright (c) 2020. Lightly AG and its affiliates.
# All Rights Reserved
import os.path
from typing import List

import hydra
import yaml
from torch.utils.hipify.hipify_python import bcolors

from lightly.active_learning.utils import BoundingBox
from lightly.cli._helpers import fix_input_path
from lightly.cli._helpers import fix_hydra_arguments
from lightly.data import LightlyDataset
from lightly.utils.cropping.crop_image_by_bounding_boxes import crop_dataset_by_bounding_boxes_and_save
from lightly.utils.cropping.read_yolo_label_file import read_yolo_label_file


def _crop_cli(cfg, is_cli_call=True):
    input_dir = cfg['input_dir']
    if input_dir and is_cli_call:
        input_dir = fix_input_path(input_dir)
    output_dir = cfg['output_dir']
    if output_dir and is_cli_call:
        output_dir = fix_input_path(output_dir)
    label_dir = cfg['label_dir']
    if label_dir and is_cli_call:
        label_dir = fix_input_path(label_dir)
    label_names_file = cfg['label_names_file']
    if label_names_file and len(label_names_file) > 0:
        if is_cli_call:
            label_names_file = fix_input_path(label_names_file)
        with open(label_names_file, 'r') as file:
            label_names_file_dict = yaml.full_load(file)
        class_names = label_names_file_dict['names']
    else:
        class_names = None


    dataset = LightlyDataset(input_dir)


    class_indices_list_list: List[List[int]] = []
    bounding_boxes_list_list: List[List[BoundingBox]] = []

    # YOLO-Specific
    for filename_image in dataset.get_filenames():
        filepath_image_base, image_extension = os.path.splitext(filename_image)
        filepath_label = os.path.join(label_dir, filename_image).replace(image_extension, '.txt')
        class_indices, bounding_boxes = read_yolo_label_file(filepath_label, float(cfg['crop_padding']))
        class_indices_list_list.append(class_indices)
        bounding_boxes_list_list.append(bounding_boxes)

    cropped_images_list_list = \
        crop_dataset_by_bounding_boxes_and_save(dataset, output_dir, bounding_boxes_list_list, class_indices_list_list, class_names)

    print(f'Cropped images are stored at: {bcolors.OKBLUE}{output_dir}{bcolors.ENDC}')
    return cropped_images_list_list


@hydra.main(**fix_hydra_arguments(config_path = 'config', config_name = 'config'))
def crop_cli(cfg):
    """Crops images into one sub-image for each object.

    Args:
        cfg:
            The default configs are loaded from the config file.
            To overwrite them please see the section on the config file
            (.config.config.yaml).

    Command-Line Args:
        input_dir:
            Path to the input directory where images are stored.
        labels_dir:
            Path to the directory where the labels are stored. There must be one label file for each image.
            The label file must have the same name as the image file, but the extension .txt.
            For example, img_123.txt for img_123.jpg. The label file must be in YOLO format.
        output_dir:
            Path to the directory where the cropped images are stored. They are stored in one directory per input image.
        crop_padding: Optional
            The additonal padding about the bounding box. This makes the crops include the context of the object.
            The padding is relative and added to the width and height.
        label_names_file: Optional
            A yaml file including the names of the classes. If it is given, the filenames of the cropped images include
            the class names instead of the class id. This file is usually included when having a dataset in yolo format.
            Example contents of such a label_names_file.yaml: "names: ['class_name_a', 'class_name_b']"


    Examples:
        >>> # Crop images and set the crop to be 20% around the bounding box
        >>> lightly-crop input_dir=data/images label_dir=data/labels output_dir=data/cropped_images crop_padding=0.2

        >>> # Crop images and use the class names in the filename
        >>> lightly-crop input_dir=data/images label_dir=data/labels output_dir=data/cropped_images label_names_file=data/data.yaml

    """
    return _crop_cli(cfg)


def entry():
    crop_cli()



# coding: utf-8

"""
    Lightly API

    Lightly.ai enables you to do self-supervised learning in an easy and intuitive way. The lightly.ai OpenAPI spec defines how one can interact with our REST API to unleash the full potential of lightly.ai  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: support@lightly.ai
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from lightly.openapi_generated.swagger_client.configuration import Configuration


class SamaTaskData(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'int',
        'url': 'RedirectedReadUrl',
        'image': 'RedirectedReadUrl',
        'lightly_file_name': 'str',
        'lightly_meta_info': 'str'
    }

    attribute_map = {
        'id': 'id',
        'url': 'url',
        'image': 'image',
        'lightly_file_name': 'lightlyFileName',
        'lightly_meta_info': 'lightlyMetaInfo'
    }

    def __init__(self, id=None, url=None, image=None, lightly_file_name=None, lightly_meta_info=None, _configuration=None):  # noqa: E501
        """SamaTaskData - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._id = None
        self._url = None
        self._image = None
        self._lightly_file_name = None
        self._lightly_meta_info = None
        self.discriminator = None

        self.id = id
        self.url = url
        if image is not None:
            self.image = image
        if lightly_file_name is not None:
            self.lightly_file_name = lightly_file_name
        if lightly_meta_info is not None:
            self.lightly_meta_info = lightly_meta_info

    @property
    def id(self):
        """Gets the id of this SamaTaskData.  # noqa: E501


        :return: The id of this SamaTaskData.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SamaTaskData.


        :param id: The id of this SamaTaskData.  # noqa: E501
        :type: int
        """
        if self._configuration.client_side_validation and id is None:
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def url(self):
        """Gets the url of this SamaTaskData.  # noqa: E501


        :return: The url of this SamaTaskData.  # noqa: E501
        :rtype: RedirectedReadUrl
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this SamaTaskData.


        :param url: The url of this SamaTaskData.  # noqa: E501
        :type: RedirectedReadUrl
        """
        if self._configuration.client_side_validation and url is None:
            raise ValueError("Invalid value for `url`, must not be `None`")  # noqa: E501

        self._url = url

    @property
    def image(self):
        """Gets the image of this SamaTaskData.  # noqa: E501


        :return: The image of this SamaTaskData.  # noqa: E501
        :rtype: RedirectedReadUrl
        """
        return self._image

    @image.setter
    def image(self, image):
        """Sets the image of this SamaTaskData.


        :param image: The image of this SamaTaskData.  # noqa: E501
        :type: RedirectedReadUrl
        """

        self._image = image

    @property
    def lightly_file_name(self):
        """Gets the lightly_file_name of this SamaTaskData.  # noqa: E501

        The original fileName of the sample. This is unique within a dataset  # noqa: E501

        :return: The lightly_file_name of this SamaTaskData.  # noqa: E501
        :rtype: str
        """
        return self._lightly_file_name

    @lightly_file_name.setter
    def lightly_file_name(self, lightly_file_name):
        """Sets the lightly_file_name of this SamaTaskData.

        The original fileName of the sample. This is unique within a dataset  # noqa: E501

        :param lightly_file_name: The lightly_file_name of this SamaTaskData.  # noqa: E501
        :type: str
        """

        self._lightly_file_name = lightly_file_name

    @property
    def lightly_meta_info(self):
        """Gets the lightly_meta_info of this SamaTaskData.  # noqa: E501


        :return: The lightly_meta_info of this SamaTaskData.  # noqa: E501
        :rtype: str
        """
        return self._lightly_meta_info

    @lightly_meta_info.setter
    def lightly_meta_info(self, lightly_meta_info):
        """Sets the lightly_meta_info of this SamaTaskData.


        :param lightly_meta_info: The lightly_meta_info of this SamaTaskData.  # noqa: E501
        :type: str
        """

        self._lightly_meta_info = lightly_meta_info

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(SamaTaskData, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SamaTaskData):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SamaTaskData):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Lightly API

    Lightly.ai enables you to do self-supervised learning in an easy and intuitive way. The lightly.ai OpenAPI spec defines how one can interact with our REST API to unleash the full potential of lightly.ai  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: support@lightly.ai
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from lightly.openapi_generated.swagger_client.configuration import Configuration


class DatasetDataEnriched(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'MongoObjectID',
        'name': 'DatasetName',
        'user_id': 'str',
        'access_type': 'SharedAccessType',
        'type': 'DatasetType',
        'img_type': 'ImageType',
        'n_samples': 'int',
        'size_in_bytes': 'int',
        'created_at': 'Timestamp',
        'last_modified_at': 'Timestamp',
        'meta_data_configuration_id': 'MongoObjectID',
        'datasources': 'list[MongoObjectID]',
        'samples': 'list[MongoObjectID]',
        'n_tags': 'int',
        'n_embeddings': 'int'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'user_id': 'userId',
        'access_type': 'accessType',
        'type': 'type',
        'img_type': 'imgType',
        'n_samples': 'nSamples',
        'size_in_bytes': 'sizeInBytes',
        'created_at': 'createdAt',
        'last_modified_at': 'lastModifiedAt',
        'meta_data_configuration_id': 'metaDataConfigurationId',
        'datasources': 'datasources',
        'samples': 'samples',
        'n_tags': 'nTags',
        'n_embeddings': 'nEmbeddings'
    }

    def __init__(self, id=None, name=None, user_id=None, access_type=None, type=None, img_type=None, n_samples=None, size_in_bytes=None, created_at=None, last_modified_at=None, meta_data_configuration_id=None, datasources=None, samples=None, n_tags=None, n_embeddings=None, _configuration=None):  # noqa: E501
        """DatasetDataEnriched - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._id = None
        self._name = None
        self._user_id = None
        self._access_type = None
        self._type = None
        self._img_type = None
        self._n_samples = None
        self._size_in_bytes = None
        self._created_at = None
        self._last_modified_at = None
        self._meta_data_configuration_id = None
        self._datasources = None
        self._samples = None
        self._n_tags = None
        self._n_embeddings = None
        self.discriminator = None

        self.id = id
        self.name = name
        self.user_id = user_id
        if access_type is not None:
            self.access_type = access_type
        self.type = type
        if img_type is not None:
            self.img_type = img_type
        self.n_samples = n_samples
        self.size_in_bytes = size_in_bytes
        self.created_at = created_at
        self.last_modified_at = last_modified_at
        if meta_data_configuration_id is not None:
            self.meta_data_configuration_id = meta_data_configuration_id
        if datasources is not None:
            self.datasources = datasources
        self.samples = samples
        self.n_tags = n_tags
        self.n_embeddings = n_embeddings

    @property
    def id(self):
        """Gets the id of this DatasetDataEnriched.  # noqa: E501


        :return: The id of this DatasetDataEnriched.  # noqa: E501
        :rtype: MongoObjectID
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this DatasetDataEnriched.


        :param id: The id of this DatasetDataEnriched.  # noqa: E501
        :type: MongoObjectID
        """
        if self._configuration.client_side_validation and id is None:
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def name(self):
        """Gets the name of this DatasetDataEnriched.  # noqa: E501


        :return: The name of this DatasetDataEnriched.  # noqa: E501
        :rtype: DatasetName
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this DatasetDataEnriched.


        :param name: The name of this DatasetDataEnriched.  # noqa: E501
        :type: DatasetName
        """
        if self._configuration.client_side_validation and name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def user_id(self):
        """Gets the user_id of this DatasetDataEnriched.  # noqa: E501

        The owner of the dataset  # noqa: E501

        :return: The user_id of this DatasetDataEnriched.  # noqa: E501
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this DatasetDataEnriched.

        The owner of the dataset  # noqa: E501

        :param user_id: The user_id of this DatasetDataEnriched.  # noqa: E501
        :type: str
        """
        if self._configuration.client_side_validation and user_id is None:
            raise ValueError("Invalid value for `user_id`, must not be `None`")  # noqa: E501

        self._user_id = user_id

    @property
    def access_type(self):
        """Gets the access_type of this DatasetDataEnriched.  # noqa: E501


        :return: The access_type of this DatasetDataEnriched.  # noqa: E501
        :rtype: SharedAccessType
        """
        return self._access_type

    @access_type.setter
    def access_type(self, access_type):
        """Sets the access_type of this DatasetDataEnriched.


        :param access_type: The access_type of this DatasetDataEnriched.  # noqa: E501
        :type: SharedAccessType
        """

        self._access_type = access_type

    @property
    def type(self):
        """Gets the type of this DatasetDataEnriched.  # noqa: E501


        :return: The type of this DatasetDataEnriched.  # noqa: E501
        :rtype: DatasetType
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this DatasetDataEnriched.


        :param type: The type of this DatasetDataEnriched.  # noqa: E501
        :type: DatasetType
        """
        if self._configuration.client_side_validation and type is None:
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501

        self._type = type

    @property
    def img_type(self):
        """Gets the img_type of this DatasetDataEnriched.  # noqa: E501


        :return: The img_type of this DatasetDataEnriched.  # noqa: E501
        :rtype: ImageType
        """
        return self._img_type

    @img_type.setter
    def img_type(self, img_type):
        """Sets the img_type of this DatasetDataEnriched.


        :param img_type: The img_type of this DatasetDataEnriched.  # noqa: E501
        :type: ImageType
        """

        self._img_type = img_type

    @property
    def n_samples(self):
        """Gets the n_samples of this DatasetDataEnriched.  # noqa: E501


        :return: The n_samples of this DatasetDataEnriched.  # noqa: E501
        :rtype: int
        """
        return self._n_samples

    @n_samples.setter
    def n_samples(self, n_samples):
        """Sets the n_samples of this DatasetDataEnriched.


        :param n_samples: The n_samples of this DatasetDataEnriched.  # noqa: E501
        :type: int
        """
        if self._configuration.client_side_validation and n_samples is None:
            raise ValueError("Invalid value for `n_samples`, must not be `None`")  # noqa: E501

        self._n_samples = n_samples

    @property
    def size_in_bytes(self):
        """Gets the size_in_bytes of this DatasetDataEnriched.  # noqa: E501


        :return: The size_in_bytes of this DatasetDataEnriched.  # noqa: E501
        :rtype: int
        """
        return self._size_in_bytes

    @size_in_bytes.setter
    def size_in_bytes(self, size_in_bytes):
        """Sets the size_in_bytes of this DatasetDataEnriched.


        :param size_in_bytes: The size_in_bytes of this DatasetDataEnriched.  # noqa: E501
        :type: int
        """
        if self._configuration.client_side_validation and size_in_bytes is None:
            raise ValueError("Invalid value for `size_in_bytes`, must not be `None`")  # noqa: E501

        self._size_in_bytes = size_in_bytes

    @property
    def created_at(self):
        """Gets the created_at of this DatasetDataEnriched.  # noqa: E501


        :return: The created_at of this DatasetDataEnriched.  # noqa: E501
        :rtype: Timestamp
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this DatasetDataEnriched.


        :param created_at: The created_at of this DatasetDataEnriched.  # noqa: E501
        :type: Timestamp
        """
        if self._configuration.client_side_validation and created_at is None:
            raise ValueError("Invalid value for `created_at`, must not be `None`")  # noqa: E501

        self._created_at = created_at

    @property
    def last_modified_at(self):
        """Gets the last_modified_at of this DatasetDataEnriched.  # noqa: E501


        :return: The last_modified_at of this DatasetDataEnriched.  # noqa: E501
        :rtype: Timestamp
        """
        return self._last_modified_at

    @last_modified_at.setter
    def last_modified_at(self, last_modified_at):
        """Sets the last_modified_at of this DatasetDataEnriched.


        :param last_modified_at: The last_modified_at of this DatasetDataEnriched.  # noqa: E501
        :type: Timestamp
        """
        if self._configuration.client_side_validation and last_modified_at is None:
            raise ValueError("Invalid value for `last_modified_at`, must not be `None`")  # noqa: E501

        self._last_modified_at = last_modified_at

    @property
    def meta_data_configuration_id(self):
        """Gets the meta_data_configuration_id of this DatasetDataEnriched.  # noqa: E501


        :return: The meta_data_configuration_id of this DatasetDataEnriched.  # noqa: E501
        :rtype: MongoObjectID
        """
        return self._meta_data_configuration_id

    @meta_data_configuration_id.setter
    def meta_data_configuration_id(self, meta_data_configuration_id):
        """Sets the meta_data_configuration_id of this DatasetDataEnriched.


        :param meta_data_configuration_id: The meta_data_configuration_id of this DatasetDataEnriched.  # noqa: E501
        :type: MongoObjectID
        """

        self._meta_data_configuration_id = meta_data_configuration_id

    @property
    def datasources(self):
        """Gets the datasources of this DatasetDataEnriched.  # noqa: E501


        :return: The datasources of this DatasetDataEnriched.  # noqa: E501
        :rtype: list[MongoObjectID]
        """
        return self._datasources

    @datasources.setter
    def datasources(self, datasources):
        """Sets the datasources of this DatasetDataEnriched.


        :param datasources: The datasources of this DatasetDataEnriched.  # noqa: E501
        :type: list[MongoObjectID]
        """

        self._datasources = datasources

    @property
    def samples(self):
        """Gets the samples of this DatasetDataEnriched.  # noqa: E501


        :return: The samples of this DatasetDataEnriched.  # noqa: E501
        :rtype: list[MongoObjectID]
        """
        return self._samples

    @samples.setter
    def samples(self, samples):
        """Sets the samples of this DatasetDataEnriched.


        :param samples: The samples of this DatasetDataEnriched.  # noqa: E501
        :type: list[MongoObjectID]
        """
        if self._configuration.client_side_validation and samples is None:
            raise ValueError("Invalid value for `samples`, must not be `None`")  # noqa: E501

        self._samples = samples

    @property
    def n_tags(self):
        """Gets the n_tags of this DatasetDataEnriched.  # noqa: E501


        :return: The n_tags of this DatasetDataEnriched.  # noqa: E501
        :rtype: int
        """
        return self._n_tags

    @n_tags.setter
    def n_tags(self, n_tags):
        """Sets the n_tags of this DatasetDataEnriched.


        :param n_tags: The n_tags of this DatasetDataEnriched.  # noqa: E501
        :type: int
        """
        if self._configuration.client_side_validation and n_tags is None:
            raise ValueError("Invalid value for `n_tags`, must not be `None`")  # noqa: E501

        self._n_tags = n_tags

    @property
    def n_embeddings(self):
        """Gets the n_embeddings of this DatasetDataEnriched.  # noqa: E501


        :return: The n_embeddings of this DatasetDataEnriched.  # noqa: E501
        :rtype: int
        """
        return self._n_embeddings

    @n_embeddings.setter
    def n_embeddings(self, n_embeddings):
        """Sets the n_embeddings of this DatasetDataEnriched.


        :param n_embeddings: The n_embeddings of this DatasetDataEnriched.  # noqa: E501
        :type: int
        """
        if self._configuration.client_side_validation and n_embeddings is None:
            raise ValueError("Invalid value for `n_embeddings`, must not be `None`")  # noqa: E501

        self._n_embeddings = n_embeddings

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(DatasetDataEnriched, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DatasetDataEnriched):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, DatasetDataEnriched):
            return True

        return self.to_dict() != other.to_dict()



""" Lightly Dataset """

# Copyright (c) 2020. Lightly AG and its affiliates.
# All Rights Reserved

import os
import bisect
import shutil
import tempfile

from PIL import Image
from typing import List, Union, Callable, Dict, Any
from torch._C import Value

import torchvision.datasets as datasets
from torchvision import transforms

from lightly.data._helpers import _load_dataset_from_folder
from lightly.data._helpers import DatasetFolder
from lightly.data._video import VideoDataset
from lightly.utils.io import check_filenames


def _get_filename_by_index(dataset, index):
    """Default function which maps the index of an image to a filename.

    """
    if isinstance(dataset, datasets.ImageFolder):
        # filename is the path of the image relative to the dataset root
        full_path = dataset.imgs[index][0]
        return os.path.relpath(full_path, dataset.root)
    elif isinstance(dataset, DatasetFolder):
        # filename is the path of the image relative to the dataset root
        full_path = dataset.samples[index][0]
        return os.path.relpath(full_path, dataset.root)
    elif isinstance(dataset, VideoDataset):
        # filename is constructed by the video dataset
        return dataset.get_filename(index)
    else:
        # dummy to prevent crashes
        return str(index)


def _ensure_dir(path):
    """Makes sure that the directory at path exists.

    """
    dirname = os.path.dirname(path)
    os.makedirs(dirname, exist_ok=True)


def _copy_image(input_dir, output_dir, filename):
    """Copies an image from the input directory to the output directory.

    """
    source = os.path.join(input_dir, filename)
    target = os.path.join(output_dir, filename)
    _ensure_dir(target)
    shutil.copyfile(source, target)


def _save_image(image, output_dir, filename, fmt):
    """Saves an image in the output directory.

    """
    target = os.path.join(output_dir, filename)
    _ensure_dir(target)
    try:
        # try to save the image with the specified format or
        # derive the format from the filename (if format=None)
        image.save(target, format=fmt)
    except ValueError:
        # could not determine format from filename
        image.save(target, format='png')


def _dump_image(dataset, output_dir, filename, index, fmt):
    """Saves a single image to the output directory.

    Will copy the image from the input directory to the output directory
    if possible. If not (e.g. for VideoDatasets), will load the image and
    then save it to the output directory with the specified format.

    """

    if isinstance(dataset, datasets.ImageFolder):
        # can safely copy the image from the input to the output directory
        _copy_image(dataset.root, output_dir, filename)
    elif isinstance(dataset, DatasetFolder):
        # can safely copy the image from the input to the output directory
        _copy_image(dataset.root, output_dir, filename)
    else:
        # need to load the image and save it to the output directory
        image, _ = dataset[index]
        _save_image(image, output_dir, filename, fmt)


class LightlyDataset:
    """Provides a uniform data interface for the embedding models.

    Should be used for all models and functions in the lightly package.
    Returns a tuple (sample, target, fname) when accessed using __getitem__.

    The LightlyDataset supports different input sources. You can use it
    on a folder of images. You can also use it on a folder with subfolders
    with images (ImageNet style). If the input_dir has subfolders,
    each subfolder gets its own target label.
    You can also work with videos (requires pyav).
    If there are multiple videos in the input_dir each video gets a different
    target label assigned. If input_dir contains images and videos
    only the videos are used.

    Can also be used in combination with the `from_torch_dataset` method
    to load a dataset offered by torchvision (e.g. cifar10).

    Parameters:
        input_dir:
            Path to directory holding the images or videos to load.
        transform:
            Image transforms (as in torchvision).
        index_to_filename:
            Function which takes the dataset and index as input and returns
            the filename of the file at the index. If None, uses default.
        filenames:
            If not None, it filters the dataset in the input directory
            by the given filenames.

    Examples:
        >>> # load a dataset consisting of images from a local folder
        >>> # mydata/
        >>> # `- img1.png
        >>> # `- img2.png
        >>> # `- ...
        >>> import lightly.data as data
        >>> dataset = data.LightlyDataset(input_dir='path/to/mydata/')
        >>> sample, target, fname = dataset[0]
        >>>
        >>> # also works with subfolders
        >>> # mydata/
        >>> # `- subfolder1
        >>> #     `- img1.png
        >>> # `- subfolder2
        >>> # ...
        >>>
        >>> # also works with videos
        >>> # mydata/
        >>> # `- video1.mp4
        >>> # `- video2.mp4
        >>> # `- ...
    """

    def __init__(self,
                 input_dir: Union[str, None],
                 transform: transforms.Compose = None,
                 index_to_filename:
                 Callable[[datasets.VisionDataset, int], str] = None,
                 filenames: List[str] = None,
                 tqdm_args: Dict[str, Any] = None,
                 num_workers_video_frame_counting: int = 0
                 ):

        # can pass input_dir=None to create an "empty" dataset
        self.input_dir = input_dir
        if filenames is not None:
            filepaths = [
                os.path.join(input_dir, filename)
                for filename in filenames
            ]
            filepaths = set(filepaths)

            def is_valid_file(filepath: str):
                return filepath in filepaths
        else:
            is_valid_file = None

        if self.input_dir is not None:
            self.dataset = _load_dataset_from_folder(
                self.input_dir,
                transform,
                is_valid_file=is_valid_file,
                tqdm_args=tqdm_args,
                num_workers_video_frame_counting=num_workers_video_frame_counting
            )
        elif transform is not None:
            raise ValueError(
                'transform must be None when input_dir is None but is '
                f'{transform}',
            )

        # initialize function to get filename of image
        self.index_to_filename = _get_filename_by_index
        if index_to_filename is not None:
            self.index_to_filename = index_to_filename

        # if created from an input directory with filenames, check if they
        # are valid
        if input_dir:
            check_filenames(self.get_filenames())

    @classmethod
    def from_torch_dataset(cls,
                           dataset,
                           transform=None,
                           index_to_filename=None):
        """Builds a LightlyDataset from a PyTorch (or torchvision) dataset.

        Args:
            dataset:
                PyTorch/torchvision dataset.
            transform:
                Image transforms (as in torchvision).
            index_to_filename:
                Function which takes the dataset and index as input and returns
                the filename of the file at the index. If None, uses default.

        Returns:
            A LightlyDataset object.

        Examples:
            >>> # load cifar10 from torchvision
            >>> import torchvision
            >>> import lightly.data as data
            >>> base = torchvision.datasets.CIFAR10(root='./')
            >>> dataset = data.LightlyDataset.from_torch_dataset(base)

        """
        # create an "empty" dataset object
        dataset_obj = cls(
            None,
            index_to_filename=index_to_filename,
        )

        # populate it with the torch dataset
        dataset_obj.dataset = dataset
        dataset_obj.transform = transform
        return dataset_obj

    def __getitem__(self, index: int):
        """Returns (sample, target, fname) of item at index.

        Args:
            index:
                Index of the queried item.

        Returns:
            The image, target, and filename of the item at index.

        """
        fname = self.index_to_filename(self.dataset, index)
        sample, target = self.dataset.__getitem__(index)

        return sample, target, fname

    def __len__(self):
        """Returns the length of the dataset.

        """
        return len(self.dataset)

    def __add__(self, other):
        """Adds another item to the dataset.

        """
        raise NotImplementedError()

    def get_filenames(self) -> List[str]:
        """Returns all filenames in the dataset.

        """
        if hasattr(self.dataset, 'get_filenames'):
            return self.dataset.get_filenames()

        list_of_filenames = []
        for index in range(len(self)):
            fname = self.index_to_filename(self.dataset, index)
            list_of_filenames.append(fname)
        return list_of_filenames

    def dump(self,
             output_dir: str,
             filenames: Union[List[str], None] = None,
             format: Union[str, None] = None):
        """Saves images in the dataset to the output directory.

        Will copy the images from the input directory to the output directory
        if possible. If not (e.g. for VideoDatasets), will load the images and
        then save them to the output directory with the specified format.

        Args:
            output_dir:
                Output directory where the image is stored.
            filenames:
                Filenames of the images to store. If None, stores all images.
            format:
                Image format. Can be any pillow image format (png, jpg, ...).
                By default we try to use the same format as the input data. If
                not possible (e.g. for videos) we dump the image 
                as a png image to prevent compression artifacts.

        """

        if self.dataset.transform is not None:
            raise RuntimeError('Cannot dump dataset which applies transforms!')

        # create directory if it doesn't exist yet
        os.makedirs(output_dir, exist_ok=True)

        # dump all the files if no filenames were passed, otherwise dump only
        # the ones referenced in the list
        if filenames is None:
            indices = [i for i in range(self.__len__())]
            filenames = self.get_filenames()
        else:
            indices = []
            filenames = sorted(filenames)
            all_filenames = self.get_filenames()
            for index, filename in enumerate(all_filenames):
                filename_index = bisect.bisect_left(filenames, filename)
                # make sure the filename exists in filenames
                if filename_index < len(filenames) and \
                        filenames[filename_index] == filename:
                    indices.append(index)

        # dump images
        for i, filename in zip(indices, filenames):
            _dump_image(self.dataset, output_dir, filename, i, fmt=format)

    def get_filepath_from_filename(self, filename: str, image: Image = None):
        """Returns the filepath given the filename of the image

        There are three cases:
            - The dataset is a regular dataset with the images in the input dir.
            - The dataset is a video dataset, thus the images have to be saved in a
              temporary folder.
            - The dataset is a torch dataset, thus the images have to be saved in a
              temporary folder.

        Args:
            filename:
                The filename of the image
            image:
                The image corresponding to the filename

        Returns:
            The filename to the image, either the existing one (case 1) or a
            newly created jpg (case 2, 3)

        """

        has_input_dir = hasattr(self, 'input_dir') and \
            isinstance(self.input_dir, str)
        if has_input_dir:
            path_to_image = os.path.join(self.input_dir, filename)
            if os.path.isfile(path_to_image):
                # the file exists, return its filepath
                return path_to_image

        if image is None:
            raise ValueError(
                'The parameter image must not be None for'
                'VideoDatasets and TorchDatasets'
            )

        # the file doesn't exist, save it as a jpg and return filepath
        folder_path = tempfile.mkdtemp()
        filepath = os.path.join(folder_path, filename) + '.jpg'
        
        if os.path.dirname(filepath):
            os.makedirs(os.path.dirname(filepath), exist_ok=True)

        image.save(filepath)
        return filepath

    @property
    def transform(self):
        """Getter for the transform of the dataset.

        """
        return self.dataset.transform

    @transform.setter
    def transform(self, t):
        """Setter for the transform of the dataset.

        """
        self.dataset.transform = t



# coding: utf-8

# flake8: noqa
"""
    Lightly API

    Lightly.ai enables you to do self-supervised learning in an easy and intuitive way. The lightly.ai OpenAPI spec defines how one can interact with our REST API to unleash the full potential of lightly.ai  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: support@lightly.ai
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

# import models into model package
from lightly.openapi_generated.swagger_client.models.access_role import AccessRole
from lightly.openapi_generated.swagger_client.models.active_learning_score_create_request import ActiveLearningScoreCreateRequest
from lightly.openapi_generated.swagger_client.models.active_learning_score_data import ActiveLearningScoreData
from lightly.openapi_generated.swagger_client.models.active_learning_score_type import ActiveLearningScoreType
from lightly.openapi_generated.swagger_client.models.active_learning_scores import ActiveLearningScores
from lightly.openapi_generated.swagger_client.models.api_error_code import ApiErrorCode
from lightly.openapi_generated.swagger_client.models.api_error_response import ApiErrorResponse
from lightly.openapi_generated.swagger_client.models.async_task_data import AsyncTaskData
from lightly.openapi_generated.swagger_client.models.category_id import CategoryId
from lightly.openapi_generated.swagger_client.models.category_name import CategoryName
from lightly.openapi_generated.swagger_client.models.configuration_data import ConfigurationData
from lightly.openapi_generated.swagger_client.models.configuration_entry import ConfigurationEntry
from lightly.openapi_generated.swagger_client.models.configuration_set_request import ConfigurationSetRequest
from lightly.openapi_generated.swagger_client.models.configuration_value_data_type import ConfigurationValueDataType
from lightly.openapi_generated.swagger_client.models.create_docker_worker_registry_entry_request import CreateDockerWorkerRegistryEntryRequest
from lightly.openapi_generated.swagger_client.models.create_entity_response import CreateEntityResponse
from lightly.openapi_generated.swagger_client.models.crop_data import CropData
from lightly.openapi_generated.swagger_client.models.custom_sample_meta_data import CustomSampleMetaData
from lightly.openapi_generated.swagger_client.models.dataset_create_request import DatasetCreateRequest
from lightly.openapi_generated.swagger_client.models.dataset_creator import DatasetCreator
from lightly.openapi_generated.swagger_client.models.dataset_data import DatasetData
from lightly.openapi_generated.swagger_client.models.dataset_data_enriched import DatasetDataEnriched
from lightly.openapi_generated.swagger_client.models.dataset_embedding_data import DatasetEmbeddingData
from lightly.openapi_generated.swagger_client.models.dataset_name import DatasetName
from lightly.openapi_generated.swagger_client.models.dataset_name_query import DatasetNameQuery
from lightly.openapi_generated.swagger_client.models.dataset_type import DatasetType
from lightly.openapi_generated.swagger_client.models.dataset_update_request import DatasetUpdateRequest
from lightly.openapi_generated.swagger_client.models.datasource_config import DatasourceConfig
from lightly.openapi_generated.swagger_client.models.datasource_config_azure import DatasourceConfigAzure
from lightly.openapi_generated.swagger_client.models.datasource_config_base import DatasourceConfigBase
from lightly.openapi_generated.swagger_client.models.datasource_config_gcs import DatasourceConfigGCS
from lightly.openapi_generated.swagger_client.models.datasource_config_lightly import DatasourceConfigLIGHTLY
from lightly.openapi_generated.swagger_client.models.datasource_config_local import DatasourceConfigLOCAL
from lightly.openapi_generated.swagger_client.models.datasource_config_obs import DatasourceConfigOBS
from lightly.openapi_generated.swagger_client.models.datasource_config_s3 import DatasourceConfigS3
from lightly.openapi_generated.swagger_client.models.datasource_config_s3_delegated_access import DatasourceConfigS3DelegatedAccess
from lightly.openapi_generated.swagger_client.models.datasource_config_verify_data import DatasourceConfigVerifyData
from lightly.openapi_generated.swagger_client.models.datasource_config_verify_data_errors import DatasourceConfigVerifyDataErrors
from lightly.openapi_generated.swagger_client.models.datasource_processed_until_timestamp_request import DatasourceProcessedUntilTimestampRequest
from lightly.openapi_generated.swagger_client.models.datasource_processed_until_timestamp_response import DatasourceProcessedUntilTimestampResponse
from lightly.openapi_generated.swagger_client.models.datasource_purpose import DatasourcePurpose
from lightly.openapi_generated.swagger_client.models.datasource_raw_samples_data import DatasourceRawSamplesData
from lightly.openapi_generated.swagger_client.models.datasource_raw_samples_data_row import DatasourceRawSamplesDataRow
from lightly.openapi_generated.swagger_client.models.datasource_raw_samples_metadata_data import DatasourceRawSamplesMetadataData
from lightly.openapi_generated.swagger_client.models.datasource_raw_samples_metadata_data_row import DatasourceRawSamplesMetadataDataRow
from lightly.openapi_generated.swagger_client.models.datasource_raw_samples_predictions_data import DatasourceRawSamplesPredictionsData
from lightly.openapi_generated.swagger_client.models.datasource_raw_samples_predictions_data_row import DatasourceRawSamplesPredictionsDataRow
from lightly.openapi_generated.swagger_client.models.dimensionality_reduction_method import DimensionalityReductionMethod
from lightly.openapi_generated.swagger_client.models.docker_license_information import DockerLicenseInformation
from lightly.openapi_generated.swagger_client.models.docker_run_artifact_create_request import DockerRunArtifactCreateRequest
from lightly.openapi_generated.swagger_client.models.docker_run_artifact_created_data import DockerRunArtifactCreatedData
from lightly.openapi_generated.swagger_client.models.docker_run_artifact_data import DockerRunArtifactData
from lightly.openapi_generated.swagger_client.models.docker_run_artifact_type import DockerRunArtifactType
from lightly.openapi_generated.swagger_client.models.docker_run_create_request import DockerRunCreateRequest
from lightly.openapi_generated.swagger_client.models.docker_run_data import DockerRunData
from lightly.openapi_generated.swagger_client.models.docker_run_log_data import DockerRunLogData
from lightly.openapi_generated.swagger_client.models.docker_run_log_entry_data import DockerRunLogEntryData
from lightly.openapi_generated.swagger_client.models.docker_run_log_level import DockerRunLogLevel
from lightly.openapi_generated.swagger_client.models.docker_run_scheduled_create_request import DockerRunScheduledCreateRequest
from lightly.openapi_generated.swagger_client.models.docker_run_scheduled_data import DockerRunScheduledData
from lightly.openapi_generated.swagger_client.models.docker_run_scheduled_priority import DockerRunScheduledPriority
from lightly.openapi_generated.swagger_client.models.docker_run_scheduled_state import DockerRunScheduledState
from lightly.openapi_generated.swagger_client.models.docker_run_scheduled_update_request import DockerRunScheduledUpdateRequest
from lightly.openapi_generated.swagger_client.models.docker_run_state import DockerRunState
from lightly.openapi_generated.swagger_client.models.docker_run_update_request import DockerRunUpdateRequest
from lightly.openapi_generated.swagger_client.models.docker_task_description import DockerTaskDescription
from lightly.openapi_generated.swagger_client.models.docker_user_stats import DockerUserStats
from lightly.openapi_generated.swagger_client.models.docker_worker_config import DockerWorkerConfig
from lightly.openapi_generated.swagger_client.models.docker_worker_config_create_request import DockerWorkerConfigCreateRequest
from lightly.openapi_generated.swagger_client.models.docker_worker_config_data import DockerWorkerConfigData
from lightly.openapi_generated.swagger_client.models.docker_worker_config_v2 import DockerWorkerConfigV2
from lightly.openapi_generated.swagger_client.models.docker_worker_config_v2_create_request import DockerWorkerConfigV2CreateRequest
from lightly.openapi_generated.swagger_client.models.docker_worker_config_v2_docker import DockerWorkerConfigV2Docker
from lightly.openapi_generated.swagger_client.models.docker_worker_config_v2_docker_corruptness_check import DockerWorkerConfigV2DockerCorruptnessCheck
from lightly.openapi_generated.swagger_client.models.docker_worker_config_v2_docker_datasource import DockerWorkerConfigV2DockerDatasource
from lightly.openapi_generated.swagger_client.models.docker_worker_config_v2_docker_object_level import DockerWorkerConfigV2DockerObjectLevel
from lightly.openapi_generated.swagger_client.models.docker_worker_config_v2_docker_stopping_condition import DockerWorkerConfigV2DockerStoppingCondition
from lightly.openapi_generated.swagger_client.models.docker_worker_config_v2_lightly import DockerWorkerConfigV2Lightly
from lightly.openapi_generated.swagger_client.models.docker_worker_config_v2_lightly_collate import DockerWorkerConfigV2LightlyCollate
from lightly.openapi_generated.swagger_client.models.docker_worker_config_v2_lightly_criterion import DockerWorkerConfigV2LightlyCriterion
from lightly.openapi_generated.swagger_client.models.docker_worker_config_v2_lightly_loader import DockerWorkerConfigV2LightlyLoader
from lightly.openapi_generated.swagger_client.models.docker_worker_config_v2_lightly_model import DockerWorkerConfigV2LightlyModel
from lightly.openapi_generated.swagger_client.models.docker_worker_config_v2_lightly_optimizer import DockerWorkerConfigV2LightlyOptimizer
from lightly.openapi_generated.swagger_client.models.docker_worker_config_v2_lightly_trainer import DockerWorkerConfigV2LightlyTrainer
from lightly.openapi_generated.swagger_client.models.docker_worker_labels import DockerWorkerLabels
from lightly.openapi_generated.swagger_client.models.docker_worker_name import DockerWorkerName
from lightly.openapi_generated.swagger_client.models.docker_worker_registry_entry_data import DockerWorkerRegistryEntryData
from lightly.openapi_generated.swagger_client.models.docker_worker_state import DockerWorkerState
from lightly.openapi_generated.swagger_client.models.docker_worker_type import DockerWorkerType
from lightly.openapi_generated.swagger_client.models.embedding2d_coordinates import Embedding2dCoordinates
from lightly.openapi_generated.swagger_client.models.embedding2d_create_request import Embedding2dCreateRequest
from lightly.openapi_generated.swagger_client.models.embedding2d_data import Embedding2dData
from lightly.openapi_generated.swagger_client.models.embedding_data import EmbeddingData
from lightly.openapi_generated.swagger_client.models.embedding_id_is_processed_body import EmbeddingIdIsProcessedBody
from lightly.openapi_generated.swagger_client.models.file_name_format import FileNameFormat
from lightly.openapi_generated.swagger_client.models.file_output_format import FileOutputFormat
from lightly.openapi_generated.swagger_client.models.filename_and_read_url import FilenameAndReadUrl
from lightly.openapi_generated.swagger_client.models.filename_and_read_urls import FilenameAndReadUrls
from lightly.openapi_generated.swagger_client.models.general_job_result import GeneralJobResult
from lightly.openapi_generated.swagger_client.models.image_type import ImageType
from lightly.openapi_generated.swagger_client.models.initial_tag_create_request import InitialTagCreateRequest
from lightly.openapi_generated.swagger_client.models.job_result_type import JobResultType
from lightly.openapi_generated.swagger_client.models.job_state import JobState
from lightly.openapi_generated.swagger_client.models.job_status_data import JobStatusData
from lightly.openapi_generated.swagger_client.models.job_status_data_result import JobStatusDataResult
from lightly.openapi_generated.swagger_client.models.job_status_meta import JobStatusMeta
from lightly.openapi_generated.swagger_client.models.job_status_upload_method import JobStatusUploadMethod
from lightly.openapi_generated.swagger_client.models.jobs_data import JobsData
from lightly.openapi_generated.swagger_client.models.label_box_data_row import LabelBoxDataRow
from lightly.openapi_generated.swagger_client.models.label_box_data_rows import LabelBoxDataRows
from lightly.openapi_generated.swagger_client.models.label_studio_task import LabelStudioTask
from lightly.openapi_generated.swagger_client.models.label_studio_task_data import LabelStudioTaskData
from lightly.openapi_generated.swagger_client.models.label_studio_tasks import LabelStudioTasks
from lightly.openapi_generated.swagger_client.models.lightly_model_v2 import LightlyModelV2
from lightly.openapi_generated.swagger_client.models.lightly_trainer_precision_v2 import LightlyTrainerPrecisionV2
from lightly.openapi_generated.swagger_client.models.mongo_object_id import MongoObjectID
from lightly.openapi_generated.swagger_client.models.object_id import ObjectId
from lightly.openapi_generated.swagger_client.models.path_safe_name import PathSafeName
from lightly.openapi_generated.swagger_client.models.prediction_singleton import PredictionSingleton
from lightly.openapi_generated.swagger_client.models.prediction_singleton_base import PredictionSingletonBase
from lightly.openapi_generated.swagger_client.models.prediction_singleton_classification import PredictionSingletonClassification
from lightly.openapi_generated.swagger_client.models.prediction_singleton_instance_segmentation import PredictionSingletonInstanceSegmentation
from lightly.openapi_generated.swagger_client.models.prediction_singleton_keypoint_detection import PredictionSingletonKeypointDetection
from lightly.openapi_generated.swagger_client.models.prediction_singleton_object_detection import PredictionSingletonObjectDetection
from lightly.openapi_generated.swagger_client.models.prediction_singleton_semantic_segmentation import PredictionSingletonSemanticSegmentation
from lightly.openapi_generated.swagger_client.models.prediction_singletons import PredictionSingletons
from lightly.openapi_generated.swagger_client.models.prediction_task_schema import PredictionTaskSchema
from lightly.openapi_generated.swagger_client.models.prediction_task_schema_category import PredictionTaskSchemaCategory
from lightly.openapi_generated.swagger_client.models.probabilities import Probabilities
from lightly.openapi_generated.swagger_client.models.read_url import ReadUrl
from lightly.openapi_generated.swagger_client.models.redirected_read_url import RedirectedReadUrl
from lightly.openapi_generated.swagger_client.models.s3_region import S3Region
from lightly.openapi_generated.swagger_client.models.s3_server_side_encryption_kms_key import S3ServerSideEncryptionKMSKey
from lightly.openapi_generated.swagger_client.models.sama_task import SamaTask
from lightly.openapi_generated.swagger_client.models.sama_task_data import SamaTaskData
from lightly.openapi_generated.swagger_client.models.sama_tasks import SamaTasks
from lightly.openapi_generated.swagger_client.models.sample_create_request import SampleCreateRequest
from lightly.openapi_generated.swagger_client.models.sample_data import SampleData
from lightly.openapi_generated.swagger_client.models.sample_data_modes import SampleDataModes
from lightly.openapi_generated.swagger_client.models.sample_meta_data import SampleMetaData
from lightly.openapi_generated.swagger_client.models.sample_partial_mode import SamplePartialMode
from lightly.openapi_generated.swagger_client.models.sample_sort_by import SampleSortBy
from lightly.openapi_generated.swagger_client.models.sample_type import SampleType
from lightly.openapi_generated.swagger_client.models.sample_update_request import SampleUpdateRequest
from lightly.openapi_generated.swagger_client.models.sample_write_urls import SampleWriteUrls
from lightly.openapi_generated.swagger_client.models.sampling_config import SamplingConfig
from lightly.openapi_generated.swagger_client.models.sampling_config_stopping_condition import SamplingConfigStoppingCondition
from lightly.openapi_generated.swagger_client.models.sampling_create_request import SamplingCreateRequest
from lightly.openapi_generated.swagger_client.models.sampling_method import SamplingMethod
from lightly.openapi_generated.swagger_client.models.score import Score
from lightly.openapi_generated.swagger_client.models.selection_config import SelectionConfig
from lightly.openapi_generated.swagger_client.models.selection_config_entry import SelectionConfigEntry
from lightly.openapi_generated.swagger_client.models.selection_config_entry_input import SelectionConfigEntryInput
from lightly.openapi_generated.swagger_client.models.selection_config_entry_strategy import SelectionConfigEntryStrategy
from lightly.openapi_generated.swagger_client.models.selection_input_predictions_name import SelectionInputPredictionsName
from lightly.openapi_generated.swagger_client.models.selection_input_type import SelectionInputType
from lightly.openapi_generated.swagger_client.models.selection_strategy_threshold_operation import SelectionStrategyThresholdOperation
from lightly.openapi_generated.swagger_client.models.selection_strategy_type import SelectionStrategyType
from lightly.openapi_generated.swagger_client.models.shared_access_config_create_request import SharedAccessConfigCreateRequest
from lightly.openapi_generated.swagger_client.models.shared_access_config_data import SharedAccessConfigData
from lightly.openapi_generated.swagger_client.models.shared_access_type import SharedAccessType
from lightly.openapi_generated.swagger_client.models.tag_active_learning_scores_data import TagActiveLearningScoresData
from lightly.openapi_generated.swagger_client.models.tag_arithmetics_operation import TagArithmeticsOperation
from lightly.openapi_generated.swagger_client.models.tag_arithmetics_request import TagArithmeticsRequest
from lightly.openapi_generated.swagger_client.models.tag_arithmetics_response import TagArithmeticsResponse
from lightly.openapi_generated.swagger_client.models.tag_bit_mask_data import TagBitMaskData
from lightly.openapi_generated.swagger_client.models.tag_bit_mask_response import TagBitMaskResponse
from lightly.openapi_generated.swagger_client.models.tag_change_data import TagChangeData
from lightly.openapi_generated.swagger_client.models.tag_create_request import TagCreateRequest
from lightly.openapi_generated.swagger_client.models.tag_creator import TagCreator
from lightly.openapi_generated.swagger_client.models.tag_data import TagData
from lightly.openapi_generated.swagger_client.models.tag_filenames_data import TagFilenamesData
from lightly.openapi_generated.swagger_client.models.tag_name import TagName
from lightly.openapi_generated.swagger_client.models.tag_update_request import TagUpdateRequest
from lightly.openapi_generated.swagger_client.models.tag_upsize_request import TagUpsizeRequest
from lightly.openapi_generated.swagger_client.models.task_name import TaskName
from lightly.openapi_generated.swagger_client.models.task_type import TaskType
from lightly.openapi_generated.swagger_client.models.team_basic_data import TeamBasicData
from lightly.openapi_generated.swagger_client.models.timestamp import Timestamp
from lightly.openapi_generated.swagger_client.models.timestamp_seconds import TimestampSeconds
from lightly.openapi_generated.swagger_client.models.trigger2d_embedding_job_request import Trigger2dEmbeddingJobRequest
from lightly.openapi_generated.swagger_client.models.update_docker_worker_registry_entry_request import UpdateDockerWorkerRegistryEntryRequest
from lightly.openapi_generated.swagger_client.models.version_number import VersionNumber
from lightly.openapi_generated.swagger_client.models.video_frame_data import VideoFrameData
from lightly.openapi_generated.swagger_client.models.write_csv_url_data import WriteCSVUrlData
